import React from 'react';
import PropTypes from 'prop-types';
import classnames from 'classnames/bind';
import { animateScroll } from 'react-scroll';
import throttle from 'lodash.throttle';
import styles from './ScrollToTop.scss';
import footerStyles from '../Footer/Footer.scss';
import ArrowIcon from '../../assets/icon-arrow-left.svg';
import varstyles from '../../styles/_vars.scss';
import getDataObj from '../../utils/getDataObj';

const cx = classnames.bind(styles);
const defaultBottom = 46;
const defaultMobileBottom = 24;
const defaultFooterHeight = parseInt(footerStyles.footerHeight, 10) + 24; // margin 24px
const defaultFooterMobileHeight = parseInt(footerStyles.footerMobileHeight, 10) - 24;
const mobileScreenSize = parseInt(varstyles.screenSizeLg, 10);

class ScrollToTop extends React.Component {
  static propTypes = {
    showOffset: PropTypes.number,
    customFooterHeight: PropTypes.number,
    hasFooter: PropTypes.bool,
    showOnMobile: PropTypes.bool,
    children: PropTypes.element,
    stickyComponent: PropTypes.element,
    size: PropTypes.oneOf(['s', 'm']),
    scrollCb: PropTypes.func,
  };

  static defaultProps = {
    showOffset: 225,
    children: undefined,
    stickyComponent: undefined,
    customFooterHeight: 0,
    hasFooter: false,
    showOnMobile: false,
    size: 's',
    scrollCb: null,
  };

  state = {
    show: false,
    bottom: defaultBottom,
  };

  componentDidMount() {
    window.addEventListener('scroll', this.handleScroll);
  }

  componentWillUnmount() {
    window.removeEventListener('scroll', this.handleScroll);
  }

  handleScroll = throttle(() => {
    const { hasFooter, customFooterHeight, showOnMobile } = this.props;
    const isMobileMode = window.innerWidth < mobileScreenSize && showOnMobile;
    let customBottom = isMobileMode ? defaultMobileBottom : defaultBottom;

    if (window.pageYOffset >= this.props.showOffset) {
      if (hasFooter || customFooterHeight) {
        const tmpFooterHeight = isMobileMode ? defaultFooterMobileHeight : defaultFooterHeight;
        const footerHeight = customFooterHeight > 0 ? customFooterHeight : tmpFooterHeight;
        const bottomOffset = isMobileMode ? defaultMobileBottom : defaultBottom;
        const diff = footerHeight - bottomOffset;
        const scrollHeight = document.body.scrollHeight;
        const offsetTop = window.innerHeight + window.pageYOffset;
        const offsetBottom = scrollHeight - offsetTop;

        if (diff >= offsetBottom) {
          customBottom = footerHeight - offsetBottom;
        }
      }

      this.setState({ show: true, bottom: customBottom });
    } else {
      this.setState({ show: false });
    }
  }, 50);

  scrollToTop = e => {
    const { scrollCb } = this.props;

    if (e) {
      e.preventDefault();
    }

    animateScroll.scrollToTop();
    if (scrollCb) {
      scrollCb();
    }
  };

  renderStickyComponent() {
    const { stickyComponent } = this.props;

    if (!stickyComponent) return null;

    return <div className={cx('sticky-wrapper')}>{stickyComponent}</div>;
  }

  renderScrollToTop() {
    const { children, showOnMobile } = this.props;
    const dataObj = getDataObj(this.props);

    if (this.state.show) {
      return children ? (
        React.cloneElement(children, { onClick: this.scrollToTop })
      ) : (
        <button
          onClick={this.scrollToTop}
          className={cx('scroll-to-top', { 'scroll-to-top--mobile': showOnMobile })}
          {...dataObj}
        >
          <img src={ArrowIcon} alt="Scroll to Top" />
        </button>
      );
    }

    return null;
  }

  render() {
    const { size } = this.props;
    const { bottom } = this.state;

    return (
      <div className={cx('fixed-wrapper', `fixed-wrapper--${size}`)} style={{ bottom: `${bottom}px` }}>
        {this.renderStickyComponent()}
        {this.renderScrollToTop()}
      </div>
    );
  }
}

export default ScrollToTop;
